<?php

namespace Drupal\drusys_landing_page_builder\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\RedirectCommand;
use Drupal\Core\Ajax\CloseDialogCommand;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
/**
 * Class DrusysEntityForm.
 */
class DrusysEntityForm extends EntityForm {

  /**
   * An entity query factory for the robot entity type.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $entityStorage;

  /**
   * Construct the RobotFormBase.
   *
   * For simple entity forms, there's no need for a constructor. Our robot form
   * base, however, requires an entity query factory to be injected into it
   * from the container. We later use this query factory to build an entity
   * query for the exists() method.
   *
   * @param \Drupal\Core\Entity\EntityStorageInterface $entity_storage
   *   An entity query factory for the robot entity type.
   */
  public function __construct(EntityStorageInterface $entity_storage) {
    $this->entityStorage = $entity_storage;
  }

  /**
   * Factory method for RobotFormBase.
   *
   * When Drupal builds this class it does not call the constructor directly.
   * Instead, it relies on this method to build the new object. Why? The class
   * constructor may take multiple arguments that are unknown to Drupal. The
   * create() method always takes one parameter -- the container. The purpose
   * of the create() method is twofold: It provides a standard way for Drupal
   * to construct the object, meanwhile it provides you a place to get needed
   * constructor parameters from the container.
   *
   * In this case, we ask the container for an entity query factory. We then
   * pass the factory to our class as a constructor parameter.
   */
  public static function create(ContainerInterface $container) {
    $form = new static($container->get('entity_type.manager')->getStorage('ds_row_presets'));
    $form->setMessenger($container->get('messenger'));
    return $form;
  }
  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    // kint($current_path = \Drupal::service('path.current')->getPath());
    $form = parent::form($form, $form_state);
    $ds_row_presets = $this->entity;
    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Preset label'),
      '#maxlength' => 255,
      '#default_value' => $ds_row_presets->label(),
      '#description' => $this->t("Preset label for the Landing page builder."),
      '#required' => TRUE,
    ];
    // ksm($ds_row_presets);
    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $ds_row_presets->id(),
      '#machine_name' => [
        'exists' => '\Drupal\drusys_landing_page_builder\Entity\DrusysEntity::load',
      ],
      '#disabled' => !$ds_row_presets->isNew(),
    ];

    $form['tabs'] = array(
      '#type' => 'vertical_tabs',
      '#default_tab' => 'edit-layouts',
      '#attributes' => ['id' => 'box-container'],
    );    
  
    $form['layouts'] = [
      '#type' => 'details',
      '#title' => $this->t('Layouts'),
      '#open' => TRUE,
      '#tree' => TRUE,
      '#group' => 'tabs',      
    ];
    $form['colors'] = [
      '#type' => 'details',
      '#title' => $this->t('Colors'),
      '#tree' => TRUE,
      '#group' => 'tabs',
    ];    
    $form['css_styles'] = [
      '#type' => 'details',
      '#title' => $this->t('Css Styles'),
      '#tree' => TRUE,
      '#group' => 'tabs',
    ];
    $form['css_classes'] = [
      '#type' => 'details',
      '#title' => $this->t('Css Classes'),
      '#tree' => TRUE,
      '#group' => 'tabs',
    ];
    // Css classes
    $css_classes = $ds_row_presets->get('css_classes');
    $form['css_classes']['row'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Row'),
      '#size' => 1024,
      '#default_value' => isset($css_classes['row']) ? $css_classes['row'] : '',        
      '#description' => 'Example: text-white text-center home-hero-banner'
    ];    
    $form['css_classes']['column_1'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Column 1'),
      '#size' => 1024,
      '#default_value' => isset($css_classes['column_1']) ? $css_classes['column_1'] : '',
      '#description' => 'Example: text-white text-center home-hero-banner'      
    ];
    $form['css_classes']['column_2'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Column 2'),
      '#size' => 1024,
      '#default_value' => isset($css_classes['column_2']) ? $css_classes['column_2'] : '',
      '#description' => 'Example: text-white text-center home-hero-banner'             
    ];
    $form['css_classes']['column_3'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Column 3'),
      '#size' => 1024,
      '#default_value' => isset($css_classes['column_3']) ? $css_classes['column_3'] : '',
      '#description' => 'Example: text-white text-center home-hero-banner'             
    ];
    // Colors
    $colors = $ds_row_presets->get('colors');
    $form['colors']['background_color'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Background color'),
      '#size' => 60,
      '#default_value' => isset($colors['background_color']) ? $colors['background_color'] : '',           
      '#attributes' => ['class' => ['color-picker']],
    ];   
    $form['colors']['overlay_color'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Overlay color'),
      '#size' => 60,
      '#default_value' => isset($colors['overlay_color']) ? $colors['overlay_color'] : '',
      '#attributes' => ['class' => ['color-picker']],      
    ];
    $form['colors']['background_gradients'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Background gradients'),
      '#default_value' => isset($colors['background_gradients']) ? $colors['background_gradients'] : '',       
    ]; 
    $form['colors']['overlay_gradients'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Overlay gradients'),
      '#default_value' => isset($colors['overlay_gradients']) ? $colors['overlay_gradients'] : '',       
    ];
    // Css styles
    $css_styles = $ds_row_presets->get('css_styles');
    $form['css_styles']['row_padding'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Row padding'),
      '#description' => 'Any valid css values. Ex: 6rem 0 or 30px 0 30px 0 or 30px 0',
      '#default_value' => isset($css_styles['row_padding']) ? $css_styles['row_padding'] : '',
    ];
  
    $form['css_styles']['row_margin'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Row margin'),
      '#description' => 'Any valid css values. Ex: 6rem 0 or 30px 0 30px 0 or 30px 0',
      '#default_value' => isset($css_styles['row_margin']) ? $css_styles['row_margin'] : '',      
    ];
  
    // Layouts
    $styles = \Drupal::entityTypeManager()->getStorage('image_style')->loadMultiple();
    $style_list['none'] = 'None (original size)';
    foreach ($styles as $key => $value) {
      $style_list[$value->getName()] = $value->label();
    }
    $layouts = $ds_row_presets->get('layouts');
    $form['layouts']['large_image_style'] = [
      '#type' => 'select',
      '#title' => $this->t('Large image size'),
      '#options' => $style_list,
      '#default_value' => isset($layouts['large_image_style']) ? $layouts['large_image_style'] : 'none',      
    ];    
    $form['layouts']['small_image_style'] = [
      '#type' => 'select',
      '#title' => $this->t('Small image size'),
      '#options' => $style_list,
      '#default_value' => isset($layouts['small_image_style']) ? $layouts['small_image_style'] : 'none',      
    ]; 
    $form['layouts']['small_image_visibility'] = [
      '#type' => 'select',
      '#title' => $this->t('Small image visibility'),
      '#options' => ['992' => 'width (0 to 992 pixels - md)', '768' => 'width (0 to 768 pixels - sm)', '576' => 'width (0 to 576 pixels - col)'],
      '#default_value' => isset($layouts['small_image_visibility']) ? $layouts['small_image_visibility'] : '576',      
    ]; 
    $form['layouts']['full_width'] = [
      '#type' => 'checkboxes',
      '#options' => [1 => 'Full width'],
      '#title' => $this->t('Width (full stretch)'),
      '#default_value' => isset($layouts['full_width']) ? $layouts['full_width'] : [0],   
    ];    
    $layout_lists = ["100" => "100%", "75" => "75%", "66" => "66%", "50" => "50%", "33" => "33%", "25" => "25%", "50_50" => "50% - 50%", "33_66" => "33% - 66%", "66_33" => "66% - 33%", "25_75" => "25% - 75%", "75_25" => "75% - 25%", "33_33_33" => "33% - 33% - 33%", "none" => 'None (I will use custom classes)'];
    $form['layouts']['width_992'] = [
      '#type' => 'radios',
      '#title' => $this->t('Columns: Width > 992 (large devices - lg)'),
      '#options' => $layout_lists,
      '#default_value' => isset($layouts['width_992']) ? $layouts['width_992'] : '100',
    ];
    $form['layouts']['width_768'] = [
      '#type' => 'radios',
      '#title' => $this->t('Columns: Width > 768 (medium devices - md)'),
      '#options' => $layout_lists,
      '#default_value' => isset($layouts['width_768']) ? $layouts['width_768'] : '100',
    ];
    $form['layouts']['width_576'] = [
      '#type' => 'radios',
      '#title' => $this->t('Columns: Width > 576 (small devices - sm)'),
      '#options' => $layout_lists,
      '#default_value' => isset($layouts['width_576']) ? $layouts['width_576'] : '100',
    ];    
    $form['layouts']['width_0'] = [
      '#type' => 'radios',
      '#title' => $this->t('Columns: Width > 0 (very small devices - col)'),
      '#options' => $layout_lists,
      '#default_value' => isset($layouts['width_0']) ? $layouts['width_0'] : '100',
    ];
    $form['layouts']['alignment'] = [
      '#type' => 'radios',
      '#title' => $this->t('Container Alignemnt'),
      '#options' => ["none" => "None", "top_left" => "Top Left", "top_center" => "Top Center", "top_right" => "Top Right", "middle_left" => "Middle Left", "middle_center" => "Middle Center", "middle_right" => "Middle Right", "bottom_left" => "Bottom Left", "bottom_center" => "Bottom Center", "bottom_right" => "Bottom Right",],
      '#default_value' => isset($layouts['alignment']) ? $layouts['alignment'] : 'none',
    ];   
    return $form;
  }

  /**
   * Function actions.
   */
  protected function actions(array $form, FormStateInterface $form_state) {
    // Get the basic actins from the base class.
    $actions = parent::actions($form, $form_state);

    // Change the submit button text.
    $actions['submit']['#value'] = $this->t('Save');

    // Return the result.
    return $actions;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $ds_row_presets = $this->entity;
    $status = $ds_row_presets->save();
    \Drupal::service('cache.render')->invalidateAll();
    switch ($status) {
      case SAVED_NEW:
        $this->messenger()->addMessage($this->t('Created the %label Row presets.', [
          '%label' => $ds_row_presets->label(),
        ]));
        break;

      default:
        $this->messenger()->addMessage($this->t('Saved the %label Row presets.', [
          '%label' => $ds_row_presets->label(),
        ]));
    }
    $form_state->setRedirectUrl($ds_row_presets->toUrl('collection'));
  }

}
